import os

if os.name == "nt":
    import win32_setctime

    def get_ctime(filepath):
        return os.stat(filepath).st_ctime

    def set_ctime(filepath, timestamp):
        if not win32_setctime.SUPPORTED:
            return

        try:
            win32_setctime.setctime(filepath, timestamp)
        except (OSError, ValueError):
            pass


elif hasattr(os.stat_result, "st_birthtime"):

    def get_ctime(filepath):
        return os.stat(filepath).st_birthtime

    def set_ctime(filepath, timestamp):
        pass


elif hasattr(os, "getxattr") and hasattr(os, "setxattr"):

    def get_ctime(filepath):
        try:
            return float(os.getxattr(filepath, b"user.loguru_crtime"))
        except OSError:
            return os.stat(filepath).st_mtime

    def set_ctime(filepath, timestamp):
        try:
            os.setxattr(filepath, b"user.loguru_crtime", str(timestamp).encode("ascii"))
        except OSError:
            pass


else:

    def get_ctime(filepath):
        return os.stat(filepath).st_mtime

    def set_ctime(filepath, timestamp):
        pass



import sys
import traceback


class ErrorInterceptor:
    def __init__(self, should_catch, handler_id):
        self._should_catch = should_catch
        self._handler_id = handler_id

    def should_catch(self):
        return self._should_catch

    def print(self, record=None, *, exception=None):
        if not sys.stderr:
            return

        if exception is None:
            type_, value, traceback_ = sys.exc_info()
        else:
            type_, value, traceback_ = (type(exception), exception, exception.__traceback__)

        try:
            sys.stderr.write("--- Logging error in Loguru Handler #%d ---\n" % self._handler_id)
            try:
                record_repr = str(record)
            except Exception:
                record_repr = "/!\\ Unprintable record /!\\"
            sys.stderr.write("Record was: %s\n" % record_repr)
            traceback.print_exception(type_, value, traceback_, None, sys.stderr)
            sys.stderr.write("--- End of logging error ---\n")
        except OSError:
            pass
        finally:
            del type_, value, traceback_



import os
import sys


def should_colorize(stream):
    if stream is None:
        return False

    if stream is sys.__stdout__ or stream is sys.__stderr__:
        if "PYCHARM_HOSTED" in os.environ:
            return True
        if os.name == "nt" and "TERM" in os.environ:
            return True

    try:
        return stream.isatty()
    except Exception:
        return False


def should_wrap(stream):
    if os.name != "nt":
        return False

    if stream is not sys.__stdout__ and stream is not sys.__stderr__:
        return False

    from colorama.win32 import winapi_test

    return winapi_test()


def wrap(stream):
    from colorama import AnsiToWin32

    return AnsiToWin32(stream, convert=True, strip=False, autoreset=False).stream



import sys
from sys import exc_info


def get_frame_fallback(n):
    try:
        raise Exception
    except Exception:
        frame = exc_info()[2].tb_frame.f_back
        for _ in range(n):
            frame = frame.f_back
        return frame


if hasattr(sys, "_getframe"):
    get_frame = sys._getframe
else:
    get_frame = get_frame_fallback
